@@ -47,7 +47,9 @@ if T.TYPE_CHECKING:
 
     from . import ModuleState
     from ..cmake import SingleTargetOptions
-    from ..interpreter import kwargs
+    from ..environment import Environment
+    from ..interpreter import Interpreter, kwargs
+    from ..interpreterbase import TYPE_kwargs, TYPE_var
 
     class WriteBasicPackageVersionFile(TypedDict):
 
@@ -118,7 +120,7 @@ class CMakeSubproject(ModuleObject):
                              'found': self.found_method,
                              })
 
-    def _args_to_info(self, args):
+    def _args_to_info(self, args: T.List[str]) -> T.Dict[str, str]:
         if len(args) != 1:
             raise InterpreterException('Exactly one argument is required.')
 
@@ -135,13 +137,13 @@ class CMakeSubproject(ModuleObject):
 
     @noKwargs
     @stringArgs
-    def get_variable(self, state, args, kwargs):
+    def get_variable(self, state: ModuleState, args: T.List[str], kwargs: TYPE_kwargs) -> TYPE_var:
         return self.subp.get_variable_method(args, kwargs)
 
     @FeatureNewKwargs('dependency', '0.56.0', ['include_type'])
     @permittedKwargs({'include_type'})
     @stringArgs
-    def dependency(self, state, args, kwargs):
+    def dependency(self, state: ModuleState, args: T.List[str], kwargs: T.Dict[str, str]) -> dependencies.Dependency:
         info = self._args_to_info(args)
         if info['func'] == 'executable':
             raise InvalidArguments(f'{args[0]} is an executable and does not support the dependency() method. Use target() instead.')
@@ -155,38 +157,38 @@ class CMakeSubproject(ModuleObject):
 
     @noKwargs
     @stringArgs
-    def include_directories(self, state, args, kwargs):
+    def include_directories(self, state: ModuleState, args: T.List[str], kwargs: TYPE_kwargs) -> build.IncludeDirs:
         info = self._args_to_info(args)
         return self.get_variable(state, [info['inc']], kwargs)
 
     @noKwargs
     @stringArgs
-    def target(self, state, args, kwargs):
+    def target(self, state: ModuleState, args: T.List[str], kwargs: TYPE_kwargs) -> build.Target:
         info = self._args_to_info(args)
         return self.get_variable(state, [info['tgt']], kwargs)
 
     @noKwargs
     @stringArgs
-    def target_type(self, state, args, kwargs):
+    def target_type(self, state: ModuleState, args: T.List[str], kwargs: TYPE_kwargs) -> str:
         info = self._args_to_info(args)
         return info['func']
 
     @noPosargs
     @noKwargs
-    def target_list(self, state, args, kwargs):
+    def target_list(self, state: ModuleState, args: TYPE_var, kwargs: TYPE_kwargs) -> T.List[str]:
         return self.cm_interpreter.target_list()
 
     @noPosargs
     @noKwargs
     @FeatureNew('CMakeSubproject.found()', '0.53.2')
-    def found_method(self, state, args, kwargs):
+    def found_method(self, state: ModuleState, args: TYPE_var, kwargs: TYPE_kwargs) -> bool:
         return self.subp is not None
 
 
 class CMakeSubprojectOptions(ModuleObject):
     def __init__(self) -> None:
         super().__init__()
-        self.cmake_options = []  # type: T.List[str]
+        self.cmake_options: T.List[str] = []
         self.target_options = TargetOptions()
 
         self.methods.update(
@@ -205,40 +207,34 @@ class CMakeSubprojectOptions(ModuleObject):
             return self.target_options[kwargs['target']]
         return self.target_options.global_options
 
+    @typed_pos_args('subproject_options.add_cmake_defines', varargs=dict)
     @noKwargs
-    def add_cmake_defines(self, state, args, kwargs) -> None:
-        self.cmake_options += cmake_defines_to_args(args)
+    def add_cmake_defines(self, state: ModuleState, args: T.Tuple[T.List[T.Dict[str, TYPE_var]]], kwargs: TYPE_kwargs) -> None:
+        self.cmake_options += cmake_defines_to_args(args[0])
 
-    @stringArgs
+    @typed_pos_args('subproject_options.set_override_option', str, str)
     @permittedKwargs({'target'})
-    def set_override_option(self, state, args, kwargs) -> None:
-        if len(args) != 2:
-            raise InvalidArguments('set_override_option takes exactly 2 positional arguments')
+    def set_override_option(self, state: ModuleState, args: T.Tuple[str, str], kwargs: TYPE_kwargs) -> None:
         self._get_opts(kwargs).set_opt(args[0], args[1])
 
+    @typed_pos_args('subproject_options.set_install', bool)
     @permittedKwargs({'target'})
-    def set_install(self, state, args, kwargs) -> None:
-        if len(args) != 1 or not isinstance(args[0], bool):
-            raise InvalidArguments('set_install takes exactly 1 boolean argument')
+    def set_install(self, state: ModuleState, args: T.Tuple[bool], kwargs: TYPE_kwargs) -> None:
         self._get_opts(kwargs).set_install(args[0])
 
-    @stringArgs
+    @typed_pos_args('subproject_options.append_compile_args', str, varargs=str, min_varargs=1)
     @permittedKwargs({'target'})
-    def append_compile_args(self, state, args, kwargs) -> None:
-        if len(args) < 2:
-            raise InvalidArguments('append_compile_args takes at least 2 positional arguments')
-        self._get_opts(kwargs).append_args(args[0], args[1:])
+    def append_compile_args(self, state: ModuleState, args: T.Tuple[str, T.List[str]], kwargs: TYPE_kwargs) -> None:
+        self._get_opts(kwargs).append_args(args[0], args[1])
 
-    @stringArgs
+    @typed_pos_args('subproject_options.append_compile_args', varargs=str, min_varargs=1)
     @permittedKwargs({'target'})
-    def append_link_args(self, state, args, kwargs) -> None:
-        if not args:
-            raise InvalidArguments('append_link_args takes at least 1 positional argument')
-        self._get_opts(kwargs).append_link_args(args)
+    def append_link_args(self, state: ModuleState, args: T.Tuple[T.List[str]], kwargs: TYPE_kwargs) -> None:
+        self._get_opts(kwargs).append_link_args(args[0])
 
     @noPosargs
     @noKwargs
-    def clear(self, state, args, kwargs) -> None:
+    def clear(self, state: ModuleState, args: TYPE_var, kwargs: TYPE_kwargs) -> None:
         self.cmake_options.clear()
         self.target_options = TargetOptions()
 
@@ -249,7 +245,7 @@ class CmakeModule(ExtensionModule):
 
     INFO = ModuleInfo('cmake', '0.50.0')
 
-    def __init__(self, interpreter):
+    def __init__(self, interpreter: Interpreter) -> None:
         super().__init__(interpreter)
         self.methods.update({
             'write_basic_package_version_file': self.write_basic_package_version_file,
@@ -258,7 +254,7 @@ class CmakeModule(ExtensionModule):
             'subproject_options': self.subproject_options,
         })
 
-    def detect_voidp_size(self, env):
+    def detect_voidp_size(self, env: Environment) -> int:
         compilers = env.coredata.compilers.host
         compiler = compilers.get('c', None)
         if not compiler:
@@ -267,9 +263,9 @@ class CmakeModule(ExtensionModule):
         if not compiler:
             raise mesonlib.MesonException('Requires a C or C++ compiler to compute sizeof(void *).')
 
-        return compiler.sizeof('void *', '', env)
+        return compiler.sizeof('void *', '', env)[0]
 
-    def detect_cmake(self, state):
+    def detect_cmake(self, state: ModuleState) -> bool:
         if self.cmake_detected:
             return True
 
@@ -301,7 +297,7 @@ class CmakeModule(ExtensionModule):
         KwargInfo('version', str, required=True),
         INSTALL_DIR_KW,
     )
-    def write_basic_package_version_file(self, state, args, kwargs: 'WriteBasicPackageVersionFile'):
+    def write_basic_package_version_file(self, state: ModuleState, args: TYPE_var, kwargs: 'WriteBasicPackageVersionFile') -> ModuleReturnValue:
         arch_independent = kwargs['arch_independent']
         compatibility = kwargs['compatibility']
         name = kwargs['name']
@@ -321,17 +317,17 @@ class CmakeModule(ExtensionModule):
 
         version_file = os.path.join(state.environment.scratch_dir, f'{name}ConfigVersion.cmake')
 
-        conf = {
-            'CVF_VERSION': (version, ''),
-            'CMAKE_SIZEOF_VOID_P': (str(self.detect_voidp_size(state.environment)), ''),
-            'CVF_ARCH_INDEPENDENT': (arch_independent, ''),
+        conf: T.Dict[str, T.Union[str, bool, int]] = {
+            'CVF_VERSION': version,
+            'CMAKE_SIZEOF_VOID_P': str(self.detect_voidp_size(state.environment)),
+            'CVF_ARCH_INDEPENDENT': arch_independent,
         }
-        mesonlib.do_conf_file(template_file, version_file, conf, 'meson')
+        mesonlib.do_conf_file(template_file, version_file, build.ConfigurationData(conf), 'meson')
 
         res = build.Data([mesonlib.File(True, state.environment.get_scratch_dir(), version_file)], pkgroot, pkgroot_name, None, state.subproject)
         return ModuleReturnValue(res, [res])
 
-    def create_package_file(self, infile, outfile, PACKAGE_RELATIVE_PATH, extra, confdata):
+    def create_package_file(self, infile: str, outfile: str, PACKAGE_RELATIVE_PATH: str, extra: str, confdata: build.ConfigurationData) -> None:
         package_init = PACKAGE_INIT_BASE.replace('@PACKAGE_RELATIVE_PATH@', PACKAGE_RELATIVE_PATH)
         package_init = package_init.replace('@inputFileName@', os.path.basename(infile))
         package_init += extra
@@ -369,7 +365,7 @@ class CmakeModule(ExtensionModule):
         KwargInfo('name', str, required=True),
         INSTALL_DIR_KW,
     )
-    def configure_package_config_file(self, state, args, kwargs: 'ConfigurePackageConfigFile'):
+    def configure_package_config_file(self, state: ModuleState, args: TYPE_var, kwargs: 'ConfigurePackageConfigFile') -> build.Data:
         inputfile = kwargs['input']
         if isinstance(inputfile, str):
             inputfile = mesonlib.File.from_source_file(state.environment.source_dir, state.subdir, inputfile)
@@ -395,7 +391,8 @@ class CmakeModule(ExtensionModule):
         if not os.path.isabs(abs_install_dir):
             abs_install_dir = os.path.join(prefix, install_dir)
 
-        PACKAGE_RELATIVE_PATH = os.path.relpath(prefix, abs_install_dir)
+        # path used in cmake scripts are POSIX even on Windows
+        PACKAGE_RELATIVE_PATH = pathlib.PurePath(os.path.relpath(prefix, abs_install_dir)).as_posix()
         extra = ''
         if re.match('^(/usr)?/lib(64)?/.+', abs_install_dir):
             extra = PACKAGE_INIT_EXT.replace('@absInstallDir@', abs_install_dir)
@@ -435,10 +432,10 @@ class CmakeModule(ExtensionModule):
             'required': kwargs_['required'],
             'options': kwargs_['options'],
             'cmake_options': kwargs_['cmake_options'],
-            'default_options': [],
+            'default_options': {},
             'version': [],
         }
-        subp = self.interpreter.do_subproject(dirname, 'cmake', kw)
+        subp = self.interpreter.do_subproject(dirname, kw, force_method='cmake')
         if not subp.found():
             return subp
         return CMakeSubproject(subp)
@@ -446,8 +443,8 @@ class CmakeModule(ExtensionModule):
     @FeatureNew('subproject_options', '0.55.0')
     @noKwargs
     @noPosargs
-    def subproject_options(self, state, args, kwargs) -> CMakeSubprojectOptions:
+    def subproject_options(self, state: ModuleState, args: TYPE_var, kwargs: TYPE_kwargs) -> CMakeSubprojectOptions:
         return CMakeSubprojectOptions()
 
-def initialize(*args, **kwargs):
+def initialize(*args: T.Any, **kwargs: T.Any) -> CmakeModule:
     return CmakeModule(*args, **kwargs)
