@@ -14,15 +14,17 @@
 from __future__ import annotations
 
 from os import path
+import shlex
 import typing as T
 
-from . import ExtensionModule, ModuleReturnValue
+from . import ExtensionModule, ModuleReturnValue, ModuleInfo
 from .. import build
 from .. import mesonlib
 from .. import mlog
-from ..interpreter.type_checking import CT_BUILD_BY_DEFAULT, CT_INPUT_KW, CT_INSTALL_DIR_KW, CT_INSTALL_TAG_KW, CT_OUTPUT_KW, INSTALL_KW, NoneType, in_set_validator
+from ..interpreter.type_checking import CT_BUILD_BY_DEFAULT, CT_INPUT_KW, INSTALL_TAG_KW, OUTPUT_KW, INSTALL_DIR_KW, INSTALL_KW, NoneType, in_set_validator
 from ..interpreterbase import FeatureNew
 from ..interpreterbase.decorators import ContainerTypeInfo, KwargInfo, noPosargs, typed_kwargs, typed_pos_args
+from ..programs import ExternalProgram
 from ..scripts.gettext import read_linguas
 
 if T.TYPE_CHECKING:
@@ -32,8 +34,6 @@ if T.TYPE_CHECKING:
     from ..build import Target
     from ..interpreter import Interpreter
     from ..interpreterbase import TYPE_var
-    from ..mparser import BaseNode
-    from ..programs import ExternalProgram
 
     class MergeFile(TypedDict):
 
@@ -41,11 +41,11 @@ if T.TYPE_CHECKING:
             str, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex,
             build.ExtractedObjects, build.GeneratedList, ExternalProgram,
             mesonlib.File]]
-        output: T.List[str]
+        output: str
         build_by_default: bool
         install: bool
-        install_dir: T.List[T.Union[str, bool]]
-        install_tag: T.List[str]
+        install_dir: T.Optional[str]
+        install_tag: T.Optional[str]
         args: T.List[str]
         data_dirs: T.List[str]
         po_dir: str
@@ -66,11 +66,11 @@ if T.TYPE_CHECKING:
             str, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex,
             build.ExtractedObjects, build.GeneratedList, ExternalProgram,
             mesonlib.File]]
-        output: T.List[str]
+        output: str
         build_by_default: bool
         install: bool
-        install_dir: T.List[T.Union[str, bool]]
-        install_tag: T.List[str]
+        install_dir: T.Optional[str]
+        install_tag: T.Optional[str]
         its_files: T.List[str]
         mo_targets: T.List[T.Union[build.BuildTarget, build.CustomTarget, build.CustomTargetIndex]]
 
@@ -125,6 +125,9 @@ PRESET_ARGS = {
 
 
 class I18nModule(ExtensionModule):
+
+    INFO = ModuleInfo('i18n')
+
     def __init__(self, interpreter: 'Interpreter'):
         super().__init__(interpreter)
         self.methods.update({
@@ -132,7 +135,7 @@ class I18nModule(ExtensionModule):
             'gettext': self.gettext,
             'itstool_join': self.itstool_join,
         })
-        self.tools: T.Dict[str, T.Optional[ExternalProgram]] = {
+        self.tools: T.Dict[str, T.Optional[T.Union[ExternalProgram, build.Executable]]] = {
             'itstool': None,
             'msgfmt': None,
             'msginit': None,
@@ -140,10 +143,6 @@ class I18nModule(ExtensionModule):
             'xgettext': None,
         }
 
-    @staticmethod
-    def nogettext_warning(location: BaseNode) -> None:
-        mlog.warning('Gettext not found, all translation targets will be ignored.', once=True, location=location)
-
     @staticmethod
     def _get_data_dirs(state: 'ModuleState', dirs: T.Iterable[str]) -> T.List[str]:
         """Returns source directories of relative paths"""
@@ -156,9 +155,9 @@ class I18nModule(ExtensionModule):
         'i18n.merge_file',
         CT_BUILD_BY_DEFAULT,
         CT_INPUT_KW,
-        CT_INSTALL_DIR_KW,
-        CT_INSTALL_TAG_KW,
-        CT_OUTPUT_KW,
+        KwargInfo('install_dir', (str, NoneType)),
+        INSTALL_TAG_KW,
+        OUTPUT_KW,
         INSTALL_KW,
         _ARGS.evolve(since='0.51.0'),
         _DATA_DIRS.evolve(since='0.41.0'),
@@ -166,8 +165,17 @@ class I18nModule(ExtensionModule):
         KwargInfo('type', str, default='xml', validator=in_set_validator({'xml', 'desktop'})),
     )
     def merge_file(self, state: 'ModuleState', args: T.List['TYPE_var'], kwargs: 'MergeFile') -> ModuleReturnValue:
-        if self.tools['msgfmt'] is None:
+        if self.tools['msgfmt'] is None or not self.tools['msgfmt'].found():
             self.tools['msgfmt'] = state.find_program('msgfmt', for_machine=mesonlib.MachineChoice.BUILD)
+        if isinstance(self.tools['msgfmt'], ExternalProgram):
+            try:
+                have_version = self.tools['msgfmt'].get_version()
+            except mesonlib.MesonException as e:
+                raise mesonlib.MesonException('i18n.merge_file requires GNU msgfmt') from e
+            want_version = '>=0.19' if kwargs['type'] == 'desktop' else '>=0.19.7'
+            if not mesonlib.version_compare(have_version, want_version):
+                msg = f'i18n.merge_file requires GNU msgfmt {want_version} to produce files of type: ' + kwargs['type'] + f' (got: {have_version})'
+                raise mesonlib.MesonException(msg)
         podir = path.join(state.build_to_src, state.subdir, kwargs['po_dir'])
 
         ddirs = self._get_data_dirs(state, kwargs['data_dirs'])
@@ -191,28 +199,32 @@ class I18nModule(ExtensionModule):
         if build_by_default is None:
             build_by_default = kwargs['install']
 
+        install_tag = [kwargs['install_tag']] if kwargs['install_tag'] is not None else None
+
         ct = build.CustomTarget(
             '',
             state.subdir,
             state.subproject,
+            state.environment,
             command,
             kwargs['input'],
-            kwargs['output'],
+            [kwargs['output']],
             build_by_default=build_by_default,
             install=kwargs['install'],
-            install_dir=kwargs['install_dir'],
-            install_tag=kwargs['install_tag'],
+            install_dir=[kwargs['install_dir']] if kwargs['install_dir'] is not None else None,
+            install_tag=install_tag,
+            description='Merging translations for {}',
         )
 
         return ModuleReturnValue(ct, [ct])
 
-    @typed_pos_args('i81n.gettext', str)
+    @typed_pos_args('i18n.gettext', str)
     @typed_kwargs(
         'i18n.gettext',
         _ARGS,
         _DATA_DIRS.evolve(since='0.36.0'),
         INSTALL_KW.evolve(default=True),
-        KwargInfo('install_dir', (str, NoneType), since='0.50.0'),
+        INSTALL_DIR_KW.evolve(since='0.50.0'),
         KwargInfo('languages', ContainerTypeInfo(list, str), default=[], listify=True),
         KwargInfo(
             'preset',
@@ -222,13 +234,18 @@ class I18nModule(ExtensionModule):
         ),
     )
     def gettext(self, state: 'ModuleState', args: T.Tuple[str], kwargs: 'Gettext') -> ModuleReturnValue:
-        for tool in ['msgfmt', 'msginit', 'msgmerge', 'xgettext']:
+        for tool, strict in [('msgfmt', True), ('msginit', False), ('msgmerge', False), ('xgettext', False)]:
             if self.tools[tool] is None:
                 self.tools[tool] = state.find_program(tool, required=False, for_machine=mesonlib.MachineChoice.BUILD)
             # still not found?
             if not self.tools[tool].found():
-                self.nogettext_warning(state.current_node)
-                return ModuleReturnValue(None, [])
+                if strict:
+                    mlog.warning('Gettext not found, all translation (po) targets will be ignored.',
+                                 once=True, location=state.current_node)
+                    return ModuleReturnValue(None, [])
+                else:
+                    mlog.warning(f'{tool!r} not found, maintainer targets will not work',
+                                 once=True, fatal=False, location=state.current_node)
         packagename = args[0]
         pkg_arg = f'--pkgname={packagename}'
 
@@ -249,13 +266,21 @@ class I18nModule(ExtensionModule):
 
         extra_arg = '--extra-args=' + '@@'.join(extra_args) if extra_args else None
 
+        source_root = path.join(state.source_root, state.root_subdir)
+        subdir = path.relpath(state.subdir, start=state.root_subdir) if state.subdir else None
+
         potargs = state.environment.get_build_command() + ['--internal', 'gettext', 'pot', pkg_arg]
+        potargs.append(f'--source-root={source_root}')
+        if subdir:
+            potargs.append(f'--subdir={subdir}')
         if datadirs:
             potargs.append(datadirs)
         if extra_arg:
             potargs.append(extra_arg)
-        potargs.append('--xgettext=' + self.tools['xgettext'].get_path())
-        pottarget = build.RunTarget(packagename + '-pot', potargs, [], state.subdir, state.subproject)
+        if self.tools['xgettext'].found():
+            potargs.append('--xgettext=' + self.tools['xgettext'].get_path())
+        pottarget = build.RunTarget(packagename + '-pot', potargs, [], state.subdir, state.subproject,
+                                    state.environment, default_env=False)
         targets.append(pottarget)
 
         install = kwargs['install']
@@ -270,7 +295,8 @@ class I18nModule(ExtensionModule):
                 f'{packagename}-{l}.mo',
                 path.join(state.subdir, l, 'LC_MESSAGES'),
                 state.subproject,
-                [self.tools['msgfmt'], '@INPUT@', '-o', '@OUTPUT@'],
+                state.environment,
+                [self.tools['msgfmt'], '-o', '@OUTPUT@', '@INPUT@'],
                 [po_file],
                 [f'{packagename}.mo'],
                 install=install,
@@ -280,14 +306,19 @@ class I18nModule(ExtensionModule):
                 # Bonus: the build tree has something usable as an uninstalled bindtextdomain() target dir.
                 install_dir=[path.join(install_dir, l, 'LC_MESSAGES')],
                 install_tag=['i18n'],
+                description='Building translation {}',
             )
             targets.append(gmotarget)
             gmotargets.append(gmotarget)
 
-        allgmotarget = build.AliasTarget(packagename + '-gmo', gmotargets, state.subdir, state.subproject)
+        allgmotarget = build.AliasTarget(packagename + '-gmo', gmotargets, state.subdir, state.subproject,
+                                         state.environment)
         targets.append(allgmotarget)
 
         updatepoargs = state.environment.get_build_command() + ['--internal', 'gettext', 'update_po', pkg_arg]
+        updatepoargs.append(f'--source-root={source_root}')
+        if subdir:
+            updatepoargs.append(f'--subdir={subdir}')
         if lang_arg:
             updatepoargs.append(lang_arg)
         if datadirs:
@@ -295,8 +326,10 @@ class I18nModule(ExtensionModule):
         if extra_arg:
             updatepoargs.append(extra_arg)
         for tool in ['msginit', 'msgmerge']:
-            updatepoargs.append(f'--{tool}=' + self.tools[tool].get_path())
-        updatepotarget = build.RunTarget(packagename + '-update-po', updatepoargs, [], state.subdir, state.subproject)
+            if self.tools[tool].found():
+                updatepoargs.append(f'--{tool}=' + self.tools[tool].get_path())
+        updatepotarget = build.RunTarget(packagename + '-update-po', updatepoargs, [], state.subdir, state.subproject,
+                                         state.environment, default_env=False)
         targets.append(updatepotarget)
 
         return ModuleReturnValue([gmotargets, pottarget, updatepotarget], targets)
@@ -307,9 +340,9 @@ class I18nModule(ExtensionModule):
         'i18n.itstool_join',
         CT_BUILD_BY_DEFAULT,
         CT_INPUT_KW,
-        CT_INSTALL_DIR_KW,
-        CT_INSTALL_TAG_KW,
-        CT_OUTPUT_KW,
+        KwargInfo('install_dir', (str, NoneType)),
+        INSTALL_TAG_KW,
+        OUTPUT_KW,
         INSTALL_KW,
         _ARGS.evolve(),
         KwargInfo('its_files', ContainerTypeInfo(list, str)),
@@ -328,11 +361,14 @@ class I18nModule(ExtensionModule):
         command: T.List[T.Union[str, build.BuildTarget, build.CustomTarget,
                                 build.CustomTargetIndex, 'ExternalProgram', mesonlib.File]] = []
         command.extend(state.environment.get_build_command())
+
+        itstool_cmd = self.tools['itstool'].get_command()
+        # TODO: python 3.8 can use shlex.join()
         command.extend([
             '--internal', 'itstool', 'join',
             '-i', '@INPUT@',
             '-o', '@OUTPUT@',
-            '--itstool=' + self.tools['itstool'].get_path(),
+            '--itstool=' + ' '.join(shlex.quote(c) for c in itstool_cmd),
         ])
         if its_files:
             for fname in its_files:
@@ -345,18 +381,22 @@ class I18nModule(ExtensionModule):
         if build_by_default is None:
             build_by_default = kwargs['install']
 
+        install_tag = [kwargs['install_tag']] if kwargs['install_tag'] is not None else None
+
         ct = build.CustomTarget(
             '',
             state.subdir,
             state.subproject,
+            state.environment,
             command,
             kwargs['input'],
-            kwargs['output'],
+            [kwargs['output']],
             build_by_default=build_by_default,
             extra_depends=mo_targets,
             install=kwargs['install'],
-            install_dir=kwargs['install_dir'],
-            install_tag=kwargs['install_tag'],
+            install_dir=[kwargs['install_dir']] if kwargs['install_dir'] is not None else None,
+            install_tag=install_tag,
+            description='Merging translations for {}',
         )
 
         return ModuleReturnValue(ct, [ct])
